import json
import redis


class QueueBackend:
    def push(self, queue_name, task):
        raise NotImplementedError

    def pop(self, queue_name, timeout=1):
        raise NotImplementedError


class RedisQueueBackend(QueueBackend):
    def __init__(self, redis_url="redis://localhost"):
        self.redis = redis.Redis.from_url(redis_url)

    def push(self, queue_name, task):
        self.redis.rpush(queue_name, json.dumps(task))

    def pop(self, queue_name, timeout=1):
        item = self.redis.blpop(queue_name, timeout=timeout)
        return json.loads(item[1]) if item else None



import threading
import logging

from pyfiq.registry import get_registry
from pyfiq.manager import mgr
from pyfiq.backends import QueueBackend, RedisQueueBackend
from pyfiq.consumer import queue_consumer

log = logging.getLogger("pyfiq.worker")


def threaded_worker(backend: QueueBackend = RedisQueueBackend()):
    log.info("Starting worker")
    mgr.backend = backend
    registry = get_registry()

    log.info(f"Loaded tasks: {registry}")
    queues = {entry["queue"] for entry in registry.values()}

    for queue in queues:
        thread = threading.Thread(target=queue_consumer, args=(queue,), daemon=True)
        thread.start()



import logging
import time

from .registry import get_registry
from .manager import mgr

log = logging.getLogger("pyfiq.consumer")


def queue_consumer(queue):
    log.info(f"Starting consumer for queue: {queue}")

    while True:
        task = mgr.backend.pop(queue)
        log.info(f"Consuming message {task} of {queue}")
        if task:
            func_name = task["func"]
            entry = get_registry().get(func_name)
            if entry:
                retval = entry["func"](*task["args"], **task["kwargs"])
                log.info(f"Executed task {task["func"]}: {retval}")
        else:
            time.sleep(0.1)



import logging

from .backends import QueueBackend

log = logging.getLogger("pyfiq.manager")


class QueueManager:
    _backend: QueueBackend = None

    @property
    def backend(self):
        return self._backend

    @backend.setter
    def backend(self, backend: QueueBackend):
        self._backend = backend
        log.info(f"Backend configured: {self._backend}")


mgr = QueueManager()



import functools
import logging

from . import registry
from .manager import mgr

log = logging.getLogger("pyfiq.producer")


def fifo(queue_name):
    def decorator(func):
        registry.register(func, queue_name)

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            task = {
                "func": func.__name__,
                "args": args,
                "kwargs": kwargs,
            }
            task_id = mgr.backend.push(queue_name, task)
            log.debug(f"Enqueued {task} (task_id={task_id})")

        return wrapper

    return decorator
